@@ -174,7 +174,7 @@ use vars qw/$VERSION @ISA @EXPORT_OK $EOS $LOC $AP $P $PAP @ABBREVIATIONS/;
 use Carp qw/cluck/;
 use English;
 
-our $VERSION = '0.27';
+our $VERSION = '0.28';
 
 # LC_CTYPE now in locale "French, Canada, codeset ISO 8859-1"
 our $LOC;
@@ -198,14 +198,13 @@ our $PAP = $P.$AP;
 
 my @PEOPLE = qw( mr mrs ms dr prof mme ms?gr sens? reps? gov attys? supt  det revd? ald);
 my @TITLE_SUFFIXES = qw(PhD jn?r sn?r esq md llb);
-my @ARMY = qw( col gen lt cmdr adm capt sgt cpl maj pte cdr);
+my @MILITARY = qw( col gen lt cmdr adm capt sgt cpl maj pte cdr);
 my @INSTITUTES = qw( dept univ assn bros);
 my @COMPANIES = qw( inc ltd co corp);
 my @PLACES =
 qw(
-	arc al ave blv?d cl ct cres dr expy? 
-	dist mt ft 
-	fw?y hwa?y la pde? pl plz rd st tce 
+	arc al ave blv?d cl ct cres dr expy? fw?y hwa?y la pde? pl plz rd st tce 
+	dist mt km in ft 	
 	Ala  Ariz Ark Cal Calif Col Colo Conn 
 	Del Fed  Fla Ga Ida Id Ill Ind Ia 
 	Kan Kans Ken Ky  La Me Md Is Mass 
@@ -218,7 +217,9 @@ my @MONTHS = qw(jan feb mar apr may jun jul aug sep oct nov dec sept);
 my @MISC = qw(no esp);
 my @LATIN = qw(vs etc al ibid sic);
 
-our @ABBREVIATIONS = (@PEOPLE, @TITLE_SUFFIXES, @ARMY, @INSTITUTES, @COMPANIES, @PLACES, @MONTHS, @MISC, @LATIN ); 
+our @ABBREVIATIONS = (@PEOPLE, @TITLE_SUFFIXES, @MILITARY, @INSTITUTES, @COMPANIES, @PLACES, @MONTHS, @MISC, @LATIN );
+my $abbreviation_regex;
+_set_abbreviations_regex();
 
 
 #==============================================================================
@@ -250,6 +251,7 @@ sub get_sentences {
 #------------------------------------------------------------------------------
 sub add_acronyms {
 	push @ABBREVIATIONS, @_;
+	_set_abbreviations_regex();
 }
 
 #------------------------------------------------------------------------------
@@ -264,6 +266,7 @@ sub get_acronyms {
 #------------------------------------------------------------------------------
 sub set_acronyms {
 	@ABBREVIATIONS=@_;
+	_set_abbreviations_regex();
 }
 
 #------------------------------------------------------------------------------
@@ -282,7 +285,9 @@ sub set_EOS {
 		cluck "Won't set \$EOS to undefined value!\n";
 		return $EOS;
 	}
-	return $EOS = $new_EOS;
+    $EOS = $new_EOS;
+    _set_abbreviations_regex();
+    return $EOS;	
 }
 
 #------------------------------------------------------------------------------
@@ -334,6 +339,13 @@ sub set_locale {
 #
 #==============================================================================
 
+# save some time by pre-compiling a regex used for working with abbreviations
+sub _set_abbreviations_regex {
+    my $abbreviations = join '|', @ABBREVIATIONS;
+    $abbreviation_regex = qr[(\b(?:$abbreviations)$PAP\s)$EOS]is;
+    return;
+}
+
 ## Please email me any suggestions for optimizing these RegExps.
 sub remove_false_end_of_sentence {
 	my ($marked_segment) = @_;
@@ -352,12 +364,15 @@ sub remove_false_end_of_sentence {
 	$marked_segment=~s/(['"]$P['"]\s+)$EOS/$1/sg;
 	## fix where abbreviations exist
 	foreach (@ABBREVIATIONS) { $marked_segment=~s/(\b$_$PAP\s)$EOS/$1/isg; }
+	$marked_segment=~s/$abbreviation_regex/$1/g;
 	
 	# don't break after quote unless its a capital letter.
 	$marked_segment=~s/(["']\s*)$EOS(\s*[[:lower:]])/$1$2/sg;
 
 	# don't break: text . . some more text.
 	$marked_segment=~s/(\s\.\s)$EOS(\s*)/$1$2/sg;
+    $marked_segment=~s/(["']\s*)$EOS(\s*[[:lower:]])/$1$2/sg;
+
 
 	$marked_segment=~s/(\s$PAP\s)$EOS/$1/sg;
 	return $marked_segment;
@@ -413,6 +428,7 @@ sub first_sentence_breaking {
 	return $text;
 }
 
+
 #==============================================================================
 #
 # Return TRUE
